import pytest

from decoda import *


def test_it_can_lookup_manufacturers():
    repo = Repo(
        Manufacturer,
        manufacturer_from_dict,
        [
            {"id": 0, "name": "First"},
            {"id": 1, "name": "Second"},
        ],
    )

    assert repo.get_by_id(0).name == "First"
    assert repo.get_by_id(1).name == "Second"


def test_it_raises_error_if_manufacturer_not_found():
    repo = Repo(Manufacturer, manufacturer_from_dict, [])
    with pytest.raises(UnknownReferenceError) as e:
        repo.get_by_id(0)
    assert str(e.value) == "Manufacturer not found for id: 0"


def test_it_can_lookup_spns():
    repo = Repo(
        SPN,
        spn_from_dict,
        [
            {
                "id": 0,
                "name": "First",
                "description": "Desc",
                "bit_length": "8",
                "units": "asocinasocin",
            },
            {
                "id": 1,
                "name": "Second",
                "description": "Desc",
                "bit_length": "8",
                "units": "asocinasocin",
            },
        ],
    )

    assert "First" == repo.get_by_id(0).name
    assert "Second" == repo.get_by_id(1).name


def test_it_raises_error_if_spn_not_found():
    repo = Repo(SPN, spn_from_dict, [])
    with pytest.raises(UnknownReferenceError) as e:
        repo.get_by_id(0)
    assert str(e.value) == "SPN not found for id: 0"


def test_it_can_lookup_pgns():
    spns = Repo(
        SPN,
        spn_from_dict,
        [
            {
                "id": 0,
                "name": "First",
                "description": "Desc",
                "bit_length": "8",
            },
            {
                "id": 1,
                "name": "Second",
                "description": "Desc",
                "bit_length": "8",
            },
        ],
    )

    repo = Repo(
        PGN,
        pgn_from_dict,
        [
            {
                "id": 0,
                "label": "First PGN",
                "description": "",
                "transmission_rate": "50 ms",
                "length": 8,
                "spns": [
                    {"id": 0, "start_pos": "1.1"},
                    {"id": 1, "start_pos": "1.3"},
                ],
            }
        ],
        spns,
    )

    assert "First PGN" == repo.get_by_id(0).name


def test_it_raises_error_if_pgn_not_found():
    repo = Repo(PGN, pgn_from_dict, [])
    with pytest.raises(UnknownReferenceError) as e:
        repo.get_by_id(0)
    assert str(e.value) == "PGN not found for id: 0"



import codecs
import os
import re

from setuptools import find_packages, setup

###############################################################################

NAME = "decoda"
PACKAGES = find_packages(where="src")
META_PATH = os.path.join("src", "decoda", "__init__.py")
KEYWORDS = ["j1939"]
PROJECT_URLS = {
    "Documentation": "https://www.decoda.cc/",
    "Source Code": "https://github.com/andrewdodd/decoda",
}
CLASSIFIERS = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Topic :: Software Development :: Libraries :: Python Modules",
]
INSTALL_REQUIRES = [
    "attrs",
]
EXTRAS_REQUIRE = {
    "sae_spec_converter": [
        # Installing pretty_j1939 instead of its dependencies, even
        # though we don't really use it directly (i.e. is it here just so
        # the copied create_j1939db_json.py file works)
        "pretty_j1939",
        "xlrd2",
        "pandas",
        "openpyxl", # Needed by pandas to read XLSX files
    ]
}

###############################################################################

HERE = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    """
    Build an absolute path from *parts* and return the contents of the
    resulting file.  Assume UTF-8 encoding.
    """
    with codecs.open(os.path.join(HERE, *parts), "rb", "utf-8") as f:
        return f.read()


META_FILE = read(META_PATH)


def find_meta(meta):
    """
    Extract __*meta*__ from META_FILE.
    """
    meta_match = re.search(
        r"^__{meta}__ = ['\"]([^'\"]*)['\"]".format(meta=meta), META_FILE, re.M
    )
    if meta_match:
        return meta_match.group(1)
    raise RuntimeError("Unable to find __{meta}__ string.".format(meta=meta))


VERSION = find_meta("version")
URL = find_meta("url")
LONG = read("README.md")


if __name__ == "__main__":
    setup(
        name=NAME,
        description=find_meta("description"),
        license=find_meta("license"),
        url=URL,
        project_urls=PROJECT_URLS,
        version=VERSION,
        author=find_meta("author"),
        author_email=find_meta("email"),
        maintainer=find_meta("author"),
        maintainer_email=find_meta("email"),
        keywords=KEYWORDS,
        long_description=LONG,
        long_description_content_type="text/markdown",
        packages=PACKAGES,
        package_dir={"": "src"},
        python_requires=">=3.6",
        zip_safe=False,
        classifiers=CLASSIFIERS,
        install_requires=INSTALL_REQUIRES,
        extras_require=EXTRAS_REQUIRE,
        include_package_data=True,
        options={"bdist_wheel": {"universal": "1"}},
        entry_points={
            "console_scripts": [
                "json_from_digital_annex=decoda.sae_spec_converter.json_from_da:main [sae_spec_converter]",
                "json_from_isobus_xlsx=decoda.sae_spec_converter.json_from_isobus_xlsx:main [sae_spec_converter]",
                "enrich_spec=decoda.sae_spec_converter.enrich_spec:main [sae_spec_converter]",
                "correct_spec=decoda.sae_spec_converter.correct_spec:main [sae_spec_converter]",
                "remove_bad_items=decoda.sae_spec_converter.remove_bad_items:main [sae_spec_converter]",
            ]
        },
    )



import pytest

from decoda import *


def test_it_has_all_industry_groups(spec):
    for idx, description in enumerate(
        [
            "Global, applies to all",
            "On-Highway Equipment",
            "Agricultural and Forestry Equipment",
            "Construction Equipment",
            "Marine",
            "Industrial-Process Control-Stationary (Gen-Sets)",
            "Reserved for future assignment by SAE",
            "Reserved for future assignment by SAE",
        ]
    ):
        assert spec.IndustryGroups.get_by_id(idx).description == description


def test_global_preferred_addresses(spec):
    assert spec.preferred_address_name(0) == "Engine #1"
    assert spec.preferred_address_name(251) == "On-Board Data Logger"
    assert spec.preferred_address_name(252) == "Reserved for Experimental Use"
    assert spec.preferred_address_name(253) == "Reserved for OEM"
    assert spec.preferred_address_name(254) == "Null Address"
    assert spec.preferred_address_name(255) == "GLOBAL (All-Any Node)"


def test_on_highway_equipment_preferred_addresses(spec):
    # test it works for ranges
    assert (
        spec.preferred_address_name(128, 1)
        == "Reserved for future assignment by SAE but available for use by self configurable ECUs"
    )
    assert (
        spec.preferred_address_name(129, 1)
        == "Reserved for future assignment by SAE but available for use by self configurable ECUs"
    )

    # test it works for specific
    assert spec.preferred_address_name(162, 1) == "Slope Sensor"

    # test it falls back to the global spec for addesses below 128
    assert spec.preferred_address_name(0, 1) == "Engine #1"


def test_it_returns_broadcast_address_for_neg_one(spec):
    assert spec.preferred_address_name(-1) == "Broadcast address"
    assert spec.preferred_address_name(-1, 1) == "Broadcast address"


def test_it_falls_back_to_global_address_if_not_specified_in_insdustry_group(
    spec,
):
    assert spec.preferred_address_name(251, 1) == "On-Board Data Logger"
    assert spec.preferred_address_name(251, 2) == "On-Board Data Logger"
    assert spec.preferred_address_name(251, 3) == "On-Board Data Logger"



import argparse
import json
import re
import sys

from decoda.spec_loader import spn_from_dict


def remove_description(item):
    if "description" in item:
        item["description"] = ""
    return item


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "input_file",
        nargs="?",
        type=argparse.FileType("r"),
        default=sys.stdout,
        help="the json file to use as input",
    )
    parser.add_argument(
        "output_file",
        nargs="?",
        type=argparse.FileType("w"),
        default=sys.stdout,
        help="the json file to use as output",
    )
    parser.add_argument("--pretty", default=False, action="store_true")
    args = parser.parse_args()

    spec = json.load(args.input_file)
    spec["PGNs"] = [
        remove_description(pgn)
        for pgn in spec["PGNs"]
        if pgn["id"] in {0, 65226, 60160, 60416}
    ]
    spns = {spn["id"] for pgn in spec["PGNs"] for spn in pgn["spns"]}
    spec["SPNs"] = [
        remove_description(spn) for spn in spec["SPNs"] if spn["id"] in spns
    ]
    spec["Manufacturers"] = []
    spec["SourceAddresses"] = {
        "Global": {
            k: v
            for k, v in spec["SourceAddresses"]["Global"].items()
            if int(k) < 5
        }
    }
    json.dump(spec, args.output_file, indent=2 if args.pretty else None)


if __name__ == "__main__":
    main()
